@@ -21,7 +21,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.23';
+$VERSION = '1.24';
 
 sub ProcessJpgFromRaw($$$);
 sub WriteJpgFromRaw($$$);
@@ -187,10 +187,13 @@ my %panasonicWhiteBalance = ( #forum9396
         Protected => 1,
         # 2 - RAW DMC-FZ8/FZ18
         # 3 - RAW DMC-L10
-        # 4 - RW2 for most other models, including G9 normal resolution and YUNEEC CGO4
+        # 4 - RW2 for most other models, including G9 in "pixel shift off" mode and YUNEEC CGO4
         #     (must add 15 to black levels for RawFormat == 4)
-        # 5 - RW2 DC-GH5s and G9 HiRes
-        # missing - DMC-LX1/FZ30/FZ50/L1/LX1/LX2
+        # 5 - RW2 DC-GH5s; G9 in "pixel shift on" mode
+        # 6 - RW2 DC-S1, DC-S1r in "pixel shift off" mode
+        # 7 - RW2 DC-S1r (and probably DC-S1, have no raw samples) in "pixel shift on" mode
+        # not used - DMC-LX1/FZ30/FZ50/L1/LX1/LX2
+        # (modes 5 and 7 are lossless)
     },
     0x2e => { #JD
         Name => 'JpgFromRaw', # (writable directory!)
@@ -497,7 +500,25 @@ my %panasonicWhiteBalance = ( #forum9396
         Writable => 'int32u',
         PrintConv => { 0 => 'No', 1 => 'Yes' },
     },
-    # 1201 - LensStyle? ref forum9394
+    # Note: LensTypeMake and LensTypeModel are combined into a Composite LensType tag
+    # defined in Olympus.pm which has the same values as Olympus:LensType
+    0x1201 => {
+        Name => 'LensTypeMake',
+        Condition => '$format eq "int16u"',
+        Writable => 'int16u',
+    },
+    0x1202 => {
+        Name => 'LensTypeModel',
+        Condition => '$format eq "int16u"',
+        Writable => 'int16u',
+        RawConv => q{
+            return undef unless $val;
+            require Image::ExifTool::Olympus; # (to load Composite LensID)
+            return $val;
+        },
+        ValueConv => '$_=sprintf("%.4x",$val); s/(..)(..)/$2 $1/; $_',
+        ValueConvInv => '$val =~ s/(..) (..)/$2$1/; hex($val)',
+    },
     0x1203 => { #4
         Name => 'FocalLengthIn35mmFormat',
         Writable => 'int16u',
